//
//     Generated by class-dump 3.5 (64 bit).
//
//     class-dump is Copyright (C) 1997-1998, 2000-2001, 2004-2013 by Steve Nygard.
//

#import <XCTest/XCApplicationMonitor.h>

@interface XCApplicationMonitor_iOS : XCApplicationMonitor
{
}

- (void)_terminateApplicationProcess:(id)arg1;
- (id)monitoredApplicationWithProcessIdentifier:(int)arg1;
- (void)_beginMonitoringApplication:(id)arg1;
- (id)init;

@end
