//
//     Generated by class-dump 3.5 (64 bit).
//
//     class-dump is Copyright (C) 1997-1998, 2000-2001, 2004-2013 by Steve Nygard.
//

#import <XCTest/XCTWaiter.h>

@interface XCTestExpectationWaiter : XCTWaiter
{
}

- (long long)wait:(double)arg1 forExpectations:(id)arg2 enforceOrder:(BOOL)arg3;
- (long long)wait:(double)arg1 forExpectations:(id)arg2;

@end

