//
//     Generated by class-dump 3.5 (64 bit).
//
//     class-dump is Copyright (C) 1997-1998, 2000-2001, 2004-2013 by Steve Nygard.
//

#import "NSObject.h"

#import "XCTRunnerAutomationSession.h"

@class NSString, NSXPCConnection;

@interface XCTRunnerAutomationSession : NSObject <XCTRunnerAutomationSession>
{
    NSXPCConnection *_connection;
}
@property NSXPCConnection *connection; // @synthesize connection=_connection;

- (id)initWithEndpoint:(id)arg1;

@end
